/*
  eagle - controls and limits the programs in execution

  Copyright (C) 2000 <Tullio Dovera tdovera@libero.it>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Send any bug reports or comments to tdovera@libero.it
*/

#include <stdio.h>
#include <pwd.h>
#include <grp.h>
#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include "rules.h"
#include "arg.h"
#include "sysLog.h"

#define MAX_RULES 64

void displayRules(void);

/* User rules data structure */
struct UserRule {
  int uid;
  char *rule;
};

static struct UserRule UserRules[MAX_RULES];


/* Group rules data structure */
struct GroupRule {
  int gid;
  char *rule;
};

static struct GroupRule GroupRules[MAX_RULES];


static char *mailto="";

char *mailTo(void) {
  return mailto;
}



int loadRules(char *filename)
{
  FILE *fd;
  char line[512];
  char name[32],rule[128];
  char variable[32],value[64];
  int nUser=0,nGroup=0;
  struct passwd *pw;
  struct group  *gr;
  char *p;
  int result;

  if ((fd=fopen(filename,"r")) == NULL) {
    perror(filename);
    exit(EXIT_FAILURE);
  }

  while (fgets(line,sizeof(line),fd)!=NULL) {
    if((p=strchr(line,'#'))!=NULL)
      *p='\0';
    if((p=strchr(line,'\n'))!=NULL)
      *p='\0';
    if((p=strchr(line,'\t'))!=NULL)
      *p=' ';

    if ((p=strchr(line,'='))!=NULL) {
      *p=' ';
      result=sscanf(line,"%s %s",variable,value);
      if ((strcmp(variable,"MAIL")==0) && (result==2)) {
	mailto=malloc(strlen(value)+1);
	strcpy(mailto,value);
      }
    }

    if ((p=strchr(line,':'))!=NULL) {
      switch(line[0]) {
      case '@':
	/* Group Rule */
	strncpy(name,line+1,p-line-1);
	name[p-line-1]='\0';
	strcpy(rule,p+1);
	gr=getgrnam(name);
	if (gr!=NULL) {
	  GroupRules[nGroup].gid=gr->gr_gid;
	  GroupRules[nGroup].rule=malloc(strlen(rule)+1); 
	  strcpy(GroupRules[nGroup].rule,rule);
	  nGroup++;
	}
	break;
      default:
	/* User Rule */
	strncpy(name,line,p-line);
	name[p-line]='\0';
	strcpy(rule,p+1);
	pw=getpwnam(name);
	if (pw!=NULL) {
	  UserRules[nUser].uid=pw->pw_uid;
	  UserRules[nUser].rule=malloc(strlen(rule)+1); 
	  strcpy(UserRules[nUser].rule,rule);
	  nUser++;
	}
      }
    }
  }
  fclose(fd);
  UserRules[nUser].uid=-1;
  GroupRules[nGroup].gid=-1;

  displayRules();

  return 0;
}


/*
  return:
  0 otherwise
  1 if UserRule is satisfied
  2 if GroupRule is satisfied
*/
int ruleOK(int uid, int gid, char *progname) {

  int j;
  char s[64];

  sprintf(s,"!%s",progname);
  for(j=0; UserRules[j].uid!=-1; j++) {
    if (UserRules[j].uid==uid) {
      if (strstr(UserRules[j].rule,s)!=NULL)
	return 0;
      
      if ((strstr(UserRules[j].rule,"ALL")!=NULL) ||
	  (strstr(UserRules[j].rule,progname)!=NULL))
	return 1;
    }
  }

  for(j=0; GroupRules[j].gid!=-1; j++) {
    if (GroupRules[j].gid==gid) {
      if (strstr(GroupRules[j].rule,s)!=NULL)
	return 0;
      
      if ((strstr(GroupRules[j].rule,"ALL")!=NULL) ||
	  (strstr(GroupRules[j].rule,progname)!=NULL))
	return 2;
    }
  }

  return 0;
}



void displayRules(void) {

  int i;
  char buffer[256];

  sprintf(buffer,"SEND Mail to '%s'",mailTo());
  sysLog(progName(),buffer);
  sysLog(progName(),"Users Rules Table");
  sysLog(progName(),"---------------");
  for(i=0; UserRules[i].uid!=-1; i++) {
    sprintf(buffer,"uid=%5d rule=%s",UserRules[i].uid,UserRules[i].rule);
    sysLog(progName(),buffer);
  }
  
  sysLog(progName(),"Groups Rules Table");
  sysLog(progName(),"----------------");
  for(i=0; GroupRules[i].gid!=-1; i++) {
    sprintf(buffer,"gid=%5d rule=%s",GroupRules[i].gid,GroupRules[i].rule);
    sysLog(progName(),buffer);
  }
}

